#include <assert.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdbool.h>
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>

#include <libgpio_sys/gpio_sys.h>
#include <libi2c_bb/i2c_bb.h>

#define ADS1115_ADDR    0x48
#define VPS             6.144 / 32768.0

static struct gpio_sys gpio_sda = {
    .pin = 4,
    .direction = OUT,
};

static struct gpio_sys gpio_scl = {
    .pin = 24,
    .direction = OUT,
};

static struct i2c_bb i2c_obj = {
    .sda = &gpio_sda,
    .scl = &gpio_scl,
};

int main(void)
{
    int res;
    uint16_t value;
    float f_value;
    char w_buffer[3], r_buffer[2];

    openlog("distance_app", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);
    printf("Trying to open I2C device.\n");

    syslog(LOG_DEBUG, "Trying to open i2c_bb.\n");
    res = i2c_open(&i2c_obj);
    if(res != EXIT_SUCCESS) {
        syslog(LOG_ERR, "Unable to open i2c_bb.\n");
        return EXIT_FAILURE;
    }
    syslog(LOG_DEBUG, "i2c_bb successfully opened.\n");

    w_buffer[0] = 0b00000001;   // config register @ address 1
    w_buffer[1] = 0b11000001;   // config value high byte
    w_buffer[2] = 0b10000111;   // config value low byte

    // begin conversation
    res = i2c_write(&i2c_obj, ADS1115_ADDR, w_buffer, 3);
    if(res != 3) {
        syslog(LOG_ERR, "Unable to write to ADS1115.\n");
        i2c_close(&i2c_obj);
        return EXIT_FAILURE;
    }

    // wait for conversation complete
    do {
        res = i2c_read(&i2c_obj, ADS1115_ADDR, w_buffer, 2);
        if(res != 2) {
            syslog(LOG_ERR, "Unable to read from ADS1115.\n");
            i2c_close(&i2c_obj);
            return EXIT_FAILURE;
        }
    } while((w_buffer[0] & 0x80) == 0);

    // read conversation result
    r_buffer[0] = 0b00000000;   /* conversation register at address 0b00000000 */
    res = i2c_write(&i2c_obj, ADS1115_ADDR, r_buffer, 1);
    if(res != 1) {
        syslog(LOG_ERR, "Unable to write to ADS1115.\n");
        i2c_close(&i2c_obj);
        return EXIT_FAILURE;
    }

    // read 2 bytes
    res = i2c_read(&i2c_obj, ADS1115_ADDR, r_buffer, 2);
    if(res != 2) {
        syslog(LOG_ERR, "Unable to read from ADS1115.\n");
        i2c_close(&i2c_obj);
        return EXIT_FAILURE;
    }
    printf("Conversation done [0x%02x:%02x].\n", r_buffer[0], r_buffer[1]);
    i2c_close(&i2c_obj);


    value = r_buffer[0] << 8 | r_buffer[1];
    if(value & 0x8000) {
        value = 0;
    }
    f_value = value * VPS;

    printf("Conversation result: %4.3fV (0x%04x).\n", f_value, value);

    return res;
}

